/*
  Macro           Test_GetCharWidthHeigth
  Author          Carlo Hogeveen
  Website         eCarlo.nl/tse
  Compatibility   Windows GUI TSE 4.0 upwards
  Version         v0.2   28 May 2022

  For other fonts than Unifont I can use TSE's GetCharWithHeight() values
  to overwrite the screen with same-sized characters.

  This creates a problem using the Unifont font in the (to be created) Uniview
  extension.

  In the macro below I try three Windows APIs to determine character width and
  height, namely
    GetTextMetrics
    GetTextExtentPoint
    GetTextExtentPoint32
  but all of them return a fixed font size different from GetCharWithHeight()
  and worse, and not changing if I resize TSE's font.
  It seems I am missing something.

*/

#define OBJ_FONT 6

dll "<user32.dll>"
  integer proc GetDC(
    integer hWnd)

  integer proc ReleaseDC(
    integer hWnd,
    integer hDC)
end

dll "<gdi32.dll>"

  integer proc CreateFont(
    integer cHeight,
    integer cWidth,
    integer cEscapement,
    integer cOrientation,
    integer cWeight,
    integer bItalic,
    integer bUnderline,
    integer bStrikeOut,
    integer iCharSet,
    integer iOutPrecision,
    integer iClipPrecision,
    integer iQuality,
    integer iPitchAndFamily,
    string  pszFaceName:cstrval) : "CreateFontA"

  integer proc SelectObject(
    integer hdc,
    integer h)

  integer proc DeleteObject(
    integer h)

  integer proc GetTextMetrics(
    integer hdc,
    var string lptm:strptr) : "GetTextMetricsA"

  integer proc GetTextExtentPoint(
    integer    hdc,
    integer    lpString,
    integer    c,
    var string lpsz:strptr) : "GetTextExtentPointA"

  integer proc GetTextExtentPoint32(
    integer    hdc,
    integer    lpString,
    integer    c,
    var string pszl:strptr) : "GetTextExtentPoint32A"

  integer proc GetTextFace(
    integer hdc,
    integer  c,
    var string lpName:strptr) : "GetTextFaceA"   // LPSTR
end

integer proc long_to_integer(string s)
  integer r = 0
  integer i = 0
  for i = 4 downto 1
    r = r * 256 + Asc(s[i])
  endfor
  return(r)
end long_to_integer

integer proc byte_to_integer(string s)
  integer r = Asc(s[1])
  return(r)
end byte_to_integer

proc Main()
  integer device_context     = 0
  integer font_flags         = 0
  integer font_handle        = 0
  integer font_height        = 0
  string  font_name    [255] = Format('':255:Chr(0))
  integer font_size          = 0
  integer font_width         = 0
  string  letter_string  [1] = 'W'
  integer letter_string_addr = 0
  string  sizes_string [255] = Format('':255:Chr(0))
  string  text_face    [255] = Format('':255:Chr(0))
  string  textmetricsa [255] = Format('':255:Chr(0))
  integer tse_char_height    = 0
  integer tse_char_width     = 0

  NewFile()
  AddLine()

  GetFont(font_name, font_size, font_flags)
  GetCharWidthHeight(tse_char_width, tse_char_height)

  AddLine('TSE - GetFont and GetCharWidthHeight properties:')
  AddLine(Format('   Name  '; font_name))
  AddLine(Format('   Sizee '; font_size))
  AddLine(Format('   Flags '; font_flags))
  AddLine(Format('   Width '; tse_char_width))
  AddLine(Format('   Height'; tse_char_height))
  AddLine()



  // Start by looking up Windows properties for the default font, just to have
  // something to compare and hopefully improve understanding.

  device_context = GetDC(GetWinHandle())

  if GetTextFace(device_context, 255, text_face)
    text_face = text_face[1: Pos(Chr(0), text_face) - 1]
    AddLine('Default font - GetTextFace property:')
    AddLine(Format('   TextFace=', text_face))
    AddLine()
  else
    Warn('ERROR: GetTextFace for default font failed.')
  endif

  if GetTextMetrics(device_context, textmetricsa)
    AddLine('Default font - GetTextMetrics properties:')
    AddLine(Format('   tmHeight='          , long_to_integer(textmetricsa[ 1:4])))
    AddLine(Format('   tmAscent='          , long_to_integer(textmetricsa[ 5:4])))
    AddLine(Format('   tmDescent='         , long_to_integer(textmetricsa[ 9:4])))
    AddLine(Format('   tmInternalLeading=' , long_to_integer(textmetricsa[13:4])))
    AddLine(Format('   tmExternalLeading=' , long_to_integer(textmetricsa[17:4])))
    AddLine(Format('   tmAveCharWidth='    , long_to_integer(textmetricsa[21:4])))
    AddLine(Format('   tmMaxCharWidth='    , long_to_integer(textmetricsa[25:4])))
    AddLine(Format('   tmWeight='          , long_to_integer(textmetricsa[29:4])))
    AddLine(Format('   tmOverhang='        , long_to_integer(textmetricsa[33:4])))
    AddLine(Format('   tmDigitizedAspectX=', long_to_integer(textmetricsa[37:4])))
    AddLine(Format('   tmDigitizedAspectY=', long_to_integer(textmetricsa[41:4])))

    AddLine(Format('   tmFirstChar='       , byte_to_integer(textmetricsa[45])))
    AddLine(Format('   tmLastChar='        , byte_to_integer(textmetricsa[46])))
    AddLine(Format('   tmDefaultChar='     , byte_to_integer(textmetricsa[47])))
    AddLine(Format('   tmBreakChar='       , byte_to_integer(textmetricsa[48])))
    AddLine(Format('   tmItalic='          , byte_to_integer(textmetricsa[49])))
    AddLine(Format('   tmUnderLined='      , byte_to_integer(textmetricsa[50])))
    AddLine(Format('   tmStruckOut='       , byte_to_integer(textmetricsa[51])))
    AddLine(Format('   tmPitchAndFamily='  , byte_to_integer(textmetricsa[52])))
    AddLine(Format('   tmCharSet='         , byte_to_integer(textmetricsa[53])))

    AddLine()
  else
    Warn('ERROR: GetTextMetrics for default font failed.')
  endif



  font_handle = CreateFont(font_height,
                           font_width,
                           0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                           font_name)
  SelectObject(device_context, font_handle)

  if GetTextMetrics(device_context, textmetricsa)
    AddLine('Created font - GetTextMetrics properties:')
    AddLine(Format('   tmHeight='          , long_to_integer(textmetricsa[ 1:4])))
    AddLine(Format('   tmAscent='          , long_to_integer(textmetricsa[ 5:4])))
    AddLine(Format('   tmDescent='         , long_to_integer(textmetricsa[ 9:4])))
    AddLine(Format('   tmInternalLeading=' , long_to_integer(textmetricsa[13:4])))
    AddLine(Format('   tmExternalLeading=' , long_to_integer(textmetricsa[17:4])))
    AddLine(Format('   tmAveCharWidth='    , long_to_integer(textmetricsa[21:4])))
    AddLine(Format('   tmMaxCharWidth='    , long_to_integer(textmetricsa[25:4])))
    AddLine(Format('   tmWeight='          , long_to_integer(textmetricsa[29:4])))
    AddLine(Format('   tmOverhang='        , long_to_integer(textmetricsa[33:4])))
    AddLine(Format('   tmDigitizedAspectX=', long_to_integer(textmetricsa[37:4])))
    AddLine(Format('   tmDigitizedAspectY=', long_to_integer(textmetricsa[41:4])))

    AddLine(Format('   tmFirstChar='       , byte_to_integer(textmetricsa[45])))
    AddLine(Format('   tmLastChar='        , byte_to_integer(textmetricsa[46])))
    AddLine(Format('   tmDefaultChar='     , byte_to_integer(textmetricsa[47])))
    AddLine(Format('   tmBreakChar='       , byte_to_integer(textmetricsa[48])))
    AddLine(Format('   tmItalic='          , byte_to_integer(textmetricsa[49])))
    AddLine(Format('   tmUnderLined='      , byte_to_integer(textmetricsa[50])))
    AddLine(Format('   tmStruckOut='       , byte_to_integer(textmetricsa[51])))
    AddLine(Format('   tmPitchAndFamily='  , byte_to_integer(textmetricsa[52])))
    AddLine(Format('   tmCharSet='         , byte_to_integer(textmetricsa[53])))

    AddLine()
  else
    Warn('ERROR: GetTextMetrics for created fofailed.')
  endif

  letter_string_addr = AdjPtr(Addr(letter_string), 2)

  if GetTextExtentPoint(device_context,
                        letter_string_addr,
                        1,
                        sizes_string)
    AddLine('Created font - GetTextExtentPoint properties:')
    AddLine(Format('   cx=', long_to_integer(sizes_string[1:4])))
    AddLine(Format('   cy=', long_to_integer(sizes_string[5:4])))
    AddLine()
  else
    Warn('ERROR: GetTextExtentPoint failed.')
  endif

  sizes_string = Format('':255:Chr(0))

  if GetTextExtentPoint32(device_context,
                          letter_string_addr,
                          1,
                          sizes_string)
    AddLine('Created font - GetTextExtentPoint32 properties:')
    AddLine(Format('   cx=', long_to_integer(sizes_string[1:4])))
    AddLine(Format('   cy=', long_to_integer(sizes_string[5:4])))
    AddLine()
  else
    Warn('ERROR: GetTextExtentPoint32 failed.')
  endif

  DeleteObject(font_handle)
  ReleaseDc(GetWinHandle(), device_context)

  BegFile()
  PurgeMacro(CurrMacroFilename())
end Main

